/*++

Copyright (c) 2011 Test Equipment Plus.

Module Name:

    SHAPI.h

Abstract:

    Native Signal Hound / USB device driver for Signal Hound Firmware version 1.2 or later
    Signal Hound API library definitions


Revision History:

    3/09/2010   JBC     Created.
    3/10/2010   JBC     Added function calls from SH_API.dll
    3/25/2010   JBC     Added additional function calls from SH_API.dll
    9/7/2011	JBC	Revised for MatLab
	
	
--*/

#pragma once

//////////////////////////////////////////////////////////
// double SHAPI_GetRBW(int FFTSize, int decimation)
// Arguments: FFT size, decimation setting
// Returns calculated estimated RBW.
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) double SHAPI_GetRBW(int FFTSize, int decimation);


//////////////////////////////////////////////////////////
// int SHAPI_Initialize()
// Initializes USB and loads cal table.
// TAKES 15 seconds to complete!!!
// Arguments: none 
// Returns 0 for success, otherwise returns error code.
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_Initialize();


//////////////////////////////////////////////////////////
// int SHAPI_SetAttenuator(double attenVal)
// Sets attenuator to nearest value of 0,5,10, or 15 dB
// Arguments: double attenVal.
// Returns 0 for success, otherwise returns error code.
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_SetAttenuator(double attenVal);


//////////////////////////////////////////////////////////
// int SHAPI_SelectExt10MHz()
// Selects external reference
// Returns 0 for success, otherwise returns error code.
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_SelectExt10MHz();


//////////////////////////////////////////////////////////
// SHAPI_Configure(double attenVal, ... )
// Sets attenuator to nearest value of 0,5,10, or 15 dB
// Selects mixer as either low band (value=0) or high band (value=1)
// Selects sensitivity as 0=low, 1=mid, or 2=high sensitivity
// Selects decimation as [1..16]
// Selects IF path as either 2.92 (value=1) or 10.7 MHz (value=0)
// Selects clock as 23.33 MHz (value=0) or 22.5 MHz (value=1) 
// Arguments: double attenVal=10, int mixerBand=1, int sensitivity=0,  
//												
// Returns 0 for success, otherwise returns error code.
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_Configure(double attenVal, int mixerBand, int sensitivity,  
												   int decimation, int useIF2_9, int ADCclock);

// Same, except processes only settings that have changed
extern "C" __declspec( dllimport ) int SHAPI_ConfigureFast(double attenVal, int mixerBand, int sensitivity,  
												   int decimation, int useIF2_9, int ADCclock);


//////////////////////////////////////////////////////////
// SHAPI_GetSlowSweep
// See Signal Hound API documentation
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_GetSlowSweep(double * dBArray, double startFreq, double stopFreq,  
													  int &returnCount, int FFTSize, 
													  int avgCount, int imageHandling);


													  
//////////////////////////////////////////////////////////
// SHAPI_GetFastSweep
// See Signal Hound API documentation
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_GetFastSweep(double * dBArray, double startFreq, double stopFreq,  
													  int &returnCount, int FFTSize, int imageHandling);

//////////////////////////////////////////////////////////
// SHAPI_GetSlowSweepCount
// Returns number of samples in proposed sweep
//////////////////////////////////////////////////////////
extern "C" __declspec( dllexport ) int SHAPI_GetSlowSweepCount(double startFreq, double stopFreq, int FFTSize);

//////////////////////////////////////////////////////////
// SHAPI_GetFastSweepCount
// Returns number of samples in proposed sweep
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_GetFastSweepCount(double startFreq, double stopFreq,int FFTSize);

//////////////////////////////////////////////////////////
// SHAPI_CyclePort
// Returns 0 for success, or error code.
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_CyclePort();

//////////////////////////////////////////////////////////
// SHAPI_GetIQDataPacket
// Returns 0 for success, or error code.
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_GetIQDataPacket(int * pIData, int * pQData, double &centerFreq, int size);


//////////////////////////////////////////////////////////
// SHAPI_Authenticate()
// Returns 1 if authentication passes, 0 if authentication fails
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_Authenticate(int vendorcode);

//////////////////////////////////////////////////////////
// int SHAPI_CopyCalTable(unsigned char * p4KTable)
// Copies cal data from DLL memory into user array.
// User MUST allocate 4096 bytes
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_CopyCalTable(unsigned char * p4KTable);

//////////////////////////////////////////////////////////
// int SHAPI_InitializeEx(unsigned char * p4KTable)
// Initializes USB with provided cal table.
// TAKES <1 second to complete
// User MUST provide 4096 bytes from SHAPI_CopyCalTable
// Returns 0 for success, otherwise returns error code.
//
//
// To use: 
//  1) Call SHAPI_Initialize
//  2) Call SHAPI_CopyCalTable
//  3) Store the 4K cal data into a convenient location
//
// To quickly load the Signal Hound thereafter: 
//  4) Call SHAPI_InitializeEx with the 4K from steps 2-3 instead of initialize
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_InitializeEx(unsigned char * p4KTable, int deviceToSelect);

//////////////////////////////////////////////////////////
// SHAPI_SetupLO(double centerFreq, int mixMode=1)
// Mix mode = 0 for low-side injection
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_SetupLO(double &centerFreq, int mixMode);

//////////////////////////////////////////////////////////
// SHAPI_StartStreamingData();
// From here, repeatedly use GetStreamingPacket
// Then call StopStreamingData before doing anything else!!!
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_StartStreamingData();

//////////////////////////////////////////////////////////
// SHAPI_StopStreamingData();
// After using GetStreamingPacket
//  call StopStreamingData before doing anything else!!!
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_StopStreamingData();

//////////////////////////////////////////////////////////
// SHAPI_GetStreamingPacket(int *bufI, int *bufQ);
// Gets 4K tokens
// Then call StopStreamingData before doing anything else!!!
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) int SHAPI_GetStreamingPacket(int *bufI, int *bufQ);

//////////////////////////////////////////////////////////
// int SHAPI_GetLastChannelPower()
// Returns calculated channel power from last SlowSweep
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) double SHAPI_GetLastChannelPower();

//////////////////////////////////////////////////////////
// int SHAPI_GetLastChannelFreq()
// Returns calculated channel frequency
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) double SHAPI_GetLastChannelFreq();

//////////////////////////////////////////////////////////
// Returns demodulated audio peak frequencies
//////////////////////////////////////////////////////////
extern "C" __declspec( dllimport ) double SHAPI_GetFMFFTPK();
extern "C" __declspec( dllimport ) double SHAPI_GetAMFFTPK();
extern "C" __declspec( dllimport ) void SHAPI_ActivateAudioFFT();
extern "C" __declspec( dllimport ) void SHAPI_DeactivateAudioFFT();

extern "C" __declspec( dllimport ) double SHAPI_GetChannelPower(double cf, int *iBigI, int *iBigQ, int count);
extern "C" __declspec( dllimport ) int SHAPI_GetIntFFT(int FFTSize, int *iBigI, int *iBigQ, double * dFFTOut);
extern "C" __declspec( dllimport ) double SHAPI_GetPhaseStep();
extern "C" __declspec( dllimport ) void SHAPI_CyclePowerOnExit();
extern "C" __declspec( dllimport ) unsigned int SHAPI_GetSerNum();
extern "C" __declspec( dllimport ) double SHAPI_GetAudioFFTSample(int idx);
extern "C" __declspec( dllimport ) float SHAPI_GetTemperature();
extern "C" __declspec( dllimport ) void SHAPI_SetPreamp(int val);

extern "C" __declspec( dllimport ) int SHAPI_SelectDevice(int deviceToSelect);
extern "C" __declspec( dllimport ) int SHAPI_InitializeNext();
extern "C" __declspec( dllimport ) void SHAPI_WriteCalTable(unsigned char *myTable);

extern "C" __declspec( dllimport ) void SHAPI_SetOscRatio(double ratio);
extern "C" __declspec( dllimport ) int  SHAPI_RunMeasurementReceiver(void * LPStruct);

extern "C" __declspec( dllimport ) int SHAPI_IsPreampAvailable();


struct MEAS_RCVR_STRUCT
{
	// *** INPUTS ***
	double	RFFrequency;	//RF frequency center
	double	AudioLPFreq;	//Audio LowPass Cutoff
	double	AudioBPFreq;	//Audio BandPass Center
	int		UseLPF;		
	int		UseBPF;	
	
	// *** OUTPUTS ***
	double	RFCounter;		//RF frequency count out
	double	AMAudioFreq;	//AF frequency count out after AM demod
	double	FMAudioFreq;	//AF frequency count out after FM demod
	double	RFAmplitude;	//dBFS

	double	FMPeakPlus;
	double	FMPeakMinus;
	double	FMRMS;

	double	AMPeakPlus;
	double	AMPeakMinus;
	double	AMRMS;

};

extern "C" __declspec( dllimport ) int  SHAPI_ProcTGSweep(double * dBArray, double startFreq, double stepSize, int count, int attenval);
extern "C" __declspec( dllimport ) int  SHAPI_SetTGFreqAtten(double freq, unsigned char atten);

extern "C" __declspec( dllimport ) void  SHAPI_TGSyncOff();
extern "C" __declspec( dllimport ) int  SHAPI_BBSPSweep(double * pData, int startfreq, int stopfreq, int stepfreq);
extern "C" __declspec( dllimport ) int  SHAPI_ProcessData(double * pAmpData, double * pFreqData, 
	int * pIData, int * pQData, double ctrfreq, int datacount, int fftsz ); 

extern "C" __declspec( dllimport ) int  SHAPI_ProcessLoopPatch(double * pAmpData, double * pFreqData, 
	int * pIDataH, int * pQDataH,int * pIDataL, int * pQDataL, double ctrfreq, int fftsz, bool rejectImage ); // max fftsz 256;
extern "C" __declspec( dllimport ) int  SHAPI_BuildIQFIR(double * pReal, double * pImag); // max fftsz 256;

extern "C" __declspec( dllimport ) int  SHAPI_ProcIQ(double freq, double * dIout, double * dQout, int * pI, int * pQ, int * pIold, int * pQold, 
	double * pFIRRe, double * pFIRIm); // max fftsz 256;


